//==========================================================
//	ARTRAY Thermograph Camera Software Developer Kit
//
//						Version 1.000		2008.7.31
//
//									(C) 2008 Artray
//==========================================================


#ifndef _CARTCAMSDK_
#define _CARTCAMSDK_

#if _MSC_VER > 1000
	#pragma once
#endif


#include <windows.h>



#define WM_GRAPHNOTIFY  WM_APP + 1		// You do not use this function at this version.
#define WM_GRAPHPAINT	WM_APP + 2		// When an image is updated, this message is sent to an application.
#define WM_ERROR	WM_APP + 3		// When error is occured, this message is sent to an application.


#define  HACAM  DWORD
#define	 MAX_CAMERA_NUM		8

enum ARTCAM_CAMERATYPE {
	ARTCAM_CAMERATYPE_320		= 1,	// ARTCAM-320-THERMO
	ARTCAM_CAMERATYPE_640		= 2,	// ARTCAM-640-THERMO
	ARTCAM_CAMERATYPE_320LAN	= 3,	//
	ARTCAM_CAMERATYPE_640LAN	= 4,	//
	ARTCAM_CAMERATYPE_320GIGE	= 5,	//
	ARTCAM_CAMERATYPE_640GIGE	= 6,	//
	ARTCAM_CAMERATYPE_320WOM	= 7,	// ARTCAM-320-THERMO-WOM
	ARTCAM_CAMERATYPE_640WOM	= 8,	// ARTCAM-640-THERMO-WOM
	ARTCAM_CAMERATYPE_320LAN2	= 9,	//
	ARTCAM_CAMERATYPE_640LAN2	= 10,	//
	ARTCAM_CAMERATYPE_320LAN3	= 11,	//
	ARTCAM_CAMERATYPE_640LAN3	= 12,	//
	ARTCAM_CAMERATYPE_640P	    = 20,	//
	ARTCAM_CAMERATYPE_384P	    = 21,	//
};


enum GIGE_CAMERA_TYPE
{
	GIGE_CAMERATYPE_150P3	=	0,
	GIGE_CAMERATYPE_274KY,
	GIGE_CAMERATYPE_625KY,
	GIGE_CAMERATYPE_036MI,
	GIGE_CAMERATYPE_500MI,
	GIGE_CAMERATYPE_320TH,
	GIGE_CAMERATYPE_640TH,
};

// Error code
enum ARTCAMSDK_ERROR {
	ARTCAMSDK_NOERROR = 0,			// Normal
	ARTCAMSDK_NOT_INITIALIZE,		// Not formatted yet
	ARTCAMSDK_DISABLEDDEVICE,		// Access to invalid device
	ARTCAMSDK_CAPTURE,				// Fail scanning
	ARTCAMSDK_CAPTURELOST,			// Disapper device
	ARTCAMSDK_PARAM,				// Incorrect argument
	ARTCAMSDK_OUTOFMEMORY,			// Not enough memory for image transffering. Or fail to secure memory.
	ARTCAMSDK_CREATETHREAD,			// Fail creating scanning thread.
	ARTCAMSDK_CAMERASET,			// Error camera (device) setting
	ARTCAMSDK_FILENOTFOUND,			// File not found
	ARTCAMSDK_UNSUPPORTED,			// This function not supported
	ARTCAMSDK_UNKNOWN,				// Abnormal termination
};



// Saving image format
enum FILESAVETYPE {
	FILETYPE_IMAGE_BITMAP = 0,	// Save pseudo color as bitmap
	FILETYPE_IMAGE_RAW,			// Save only pseudo color data
	FILETYPE_IMAGE_JPEG_HIGH,	// Save pseudo color as JPEG (High resolution)
	FILETYPE_IMAGE_JPEG_NOMAL,	// Save pseudo color as JPEG (Normal)
	FILETYPE_IMAGE_JPEG_LOW,	// Save pseudo color as JPEG (Low resolution)
	FILETYPE_IMAGE_PNG,			// Save pseudo color as PNG
	FILETYPE_IMAGE_TIFF,		// Save pseudo color as TIFF

	FILETYPE_DATA_RAW,			// Save sensor data
	FILETYPE_DATA_PNG,			// Save sensor data as PNG
	FILETYPE_DATA_TIFF,			// Save sensor data as TIFF

	FILETYPE_T_CHAR,			// Save temperature data as char
	FILETYPE_T_BYTE,			// Save temperature data as unsigned char
	FILETYPE_T_LONG,			// Save temperature data as long
	FILETYPE_T_FLOAT,			// Save temperature data as float
};


// Temperature range
enum RANGE {
	RANGE_AUTO = 0,				// Auto
	RANGE_LO,					// L Range -40`150
	RANGE_HI,					// H Range   0`540
};

enum RECMODE {
	REC_NONE = 0,				// Do not save slide images
	REC_DATA,					// Save sensor data as RAW
	REC_IMAGE,					// Save pseudo color image as AVI
	REC_T,						// Save temperature data as RAW
};

// 
enum ARTCAM_PARAM {
	PARAM_NONE = 0,				// 

	// Scan only
	PARAM_WIDTH,				// Image width
	PARAM_HEIGHT,				// Image height
	PARAM_FPS,					// Current frame rate (at magnification of 100)
	PARAM_RECMODE,				// Recording mode
	PARAM_PREVIEW,				// TRUE if slide images are scanning
	PARAM_PAUSE,				// TRUE if scanning is paused
	PARAM_REC,					// TRUE if recording is on

	// Valid reading and writing
	PARAM_RINGCOUNT,			// Inside ring buffer size
	PARAM_WAITTIME,				// Waiting time between frame
	PARAM_WAITNUC,				// Pause scanning time after NUC (mm/sec.)
	PARAM_AVIMAXSIZE,			// Saving size of AVI
	PARAM_COLOR_AUTO,			// Auto adjustment of pseudo color (This setting effect from next frame)
	PARAM_COLOR_MIN,			// Show minimum value of pseudo color (Black when value is below thisj
	PARAM_COLOR_MAX,			// Show maximum value of pseudo color (White when value is below thisj
	PARAM_NUC_AUTO,				// Auto NUC ON/OFF
	PARAM_RANGE,				// Temperature range
	PARAM_SHOW_MIRROR,			// Upside down
	PARAM_DATA_MIRROR,			// Upside down
};


/***********************************************************
						Instruction
/***********************************************************

	PARAM_WIDTH
		Get image width
		This numeric value is used for creating a field of receiving data
		320 at ARTCAM_CAMERATYPE_320
		get		
		set		~

	PARAM_HEIGHT
		Get image height
		This numeric value is used for creating a field of receiving data
		240 at ARTCAM_CAMERATYPE_320
		get		
		set		~

	PARAM_FPS
		Numerical value of magnification of 100 of current frame rate
		If current frame rate is 30FPS, it becomes 3000
		get		
		set		~

	PARAM_RECMODE
		Recording Mode
		Not for actual recording. It returns registered value at Rec
		get		
		set		~

	PARAM_PREVIEW
		TRUE if currently scanning
		FALSE if scan is stopped
		get		
		set		~

	PARAM_PAUSE
		TRUE if currently scanning is paused
		FALSE is device is stopped during scanning
		get		
		set		~

	PARAM_REC
		TRUE if currently recording
		FALSE if not recording
		get		
		set		~


	PARAM_RINGCOUNT
		Specify ring buffer size of the inside
		You need to change is before Initialize
		Default 10
		get		
		set		
		Effective value		above 1

	PARAM_WAITTIME
		Waiting time between frame
		Bigger numerical value slow down frame rate
		Smaller numerical value does not make frame rate over camera's one
		Default 10
		get		
		set		
		Effective value		above 1

	PARAM_WAITNUC
		Temporary stop scanning becasue image is jammed after NUC
		Specify this time by mm/sec.
		Default 1000i1 sec.j
		get		
		set		
		Effective value		above 0

	PARAM_AVIMAXSIZE
		Limited size to save in 1 file by recording mode(Unit MB)
		If size is over limited
		data.avi  data_1.avi  data_2.avi . . .
		take consecutive name and save as another file as above
		Depend on computer spec, above 2GB AVI data would not work
		4GB is maximum for FAT32 format drive
		Default 2000(MB) = 2GB
		get		
		set		
		Effective value	P` Readable Size by file system

	PARAM_COLOR_AUTO
		Specify number of auto adjustment times of pseudo colors
		Adjust this setting from next frame
		Reflect to an image from next frame after adjustment
		0 do not work
		Above 1 Adjust specified numbers
		(This value is reduced whenever frame is updated, and auto is released afte value becomes 0)
		Adjust every frame if value is below 0
		get		
		set		
		Effective value		All

	PARAM_COLOR_MIN
		Minimum value to show pseudo color
		Becomes black below set numeric value
		get		
		set		
		Effective value		Below 0-65535 & PARAM_COLOR_MAX

	PARAM_COLOR_MAX
		Maximum value to show pseudo color
		Becomes white above set numeric value
		get		
		set		
		Effective value		Above 0-65535 & PARAM_COLOR_MIN

	PARAM_NUC_AUTO
		Swtch ON/OFF of auto NUC
		get		
		set		
		Effective value		TRUE, FALSE

	PARAM_RANGE
		Set temperature range
		get		
		set		
		Effective value		RANGE_AUTO, RANGE_LO, RANGE_HI

	PARAM_SHOW_MIRROR
		Upside down received data from GetColor,GetRaw,GetTemprature
		get		
		set		
		Effective value		TRUE, FALSE

	PARAM_DATA_MIRROR
		Upside down saved data by Save function
		get		
		set		
		Effective value		TRUE, FALSE


***********************************************************/




typedef DWORD		(*AT_GETDLLVERSION)			(void);
typedef LONG		(*AT_GETLASTERROR)			(HACAM hACam);

typedef HACAM		(*AT_INITIALIZE)			(HWND hWnd);
typedef BOOL		(*AT_RELEASE)				(HACAM hACam);

typedef BOOL		(*AT_SETWINDOWHANDLE)		(HACAM hACam, HWND hWnd);
typedef BOOL		(*AT_SETCAMERATYPE)			(HACAM hACam, ARTCAM_CAMERATYPE Type);
typedef BOOL		(*AT_SETCAMERANUMBER)		(HACAM hACam, LONG Number);
typedef BOOL		(*AT_ISCAMERAENABLE)		(HACAM hACam, LONG index, LPTSTR szDeviceName, LONG nSize);

typedef BOOL		(*AT_SETPARAM)				(HACAM hACam, ARTCAM_PARAM Param, LONG Value);
typedef LONG		(*AT_GETPARAM)				(HACAM hACam, ARTCAM_PARAM Param);

typedef BOOL		(*AT_PREVIEW)				(HACAM hACam);
typedef BOOL		(*AT_CLOSE)					(HACAM hACam);
typedef BOOL		(*AT_START)					(HACAM hACam);
typedef BOOL		(*AT_PAUSE)					(HACAM hACam);

typedef DWORD		(*AT_ONESHOT)				(HACAM hACam);
typedef BOOL		(*AT_SAVE)					(HACAM hACam, LPCTSTR szFileName, FILESAVETYPE FileType);
typedef BOOL		(*AT_REC)					(HACAM hACam, LPCTSTR szAviName, RECMODE Mode);

typedef DWORD		(*AT_GETNEWID)				(HACAM hACam);
typedef BOOL		(*AT_GETRAW)				(HACAM hACam, DWORD ID, LPBYTE pData, DWORD Size);
typedef BOOL		(*AT_GETTEMPERATURE)		(HACAM hACam, DWORD ID, float* pData, DWORD Size);
typedef BOOL		(*AT_GETCOLOR)				(HACAM hACam, DWORD ID, LPBYTE pImage, DWORD Size);

typedef BOOL		(*AT_NUC)					(HACAM hACam);
typedef BOOL		(*AT_SETTABLE)				(HACAM hACam, LPCTSTR szFileName);
typedef BOOL		(*AT_SETIPADDRESS)			(HACAM hACam, BYTE Add1, BYTE Add2, BYTE Add3, BYTE Add4);

typedef BOOL		(*AT_SETDEVICENUMBER)		(HACAM hACam, LONG Number);
typedef BOOL		(*AT_CONNECTTOCAMERA)		(HACAM hACam, BYTE Address[6]);

typedef LONG		(*AT_ENUMDEVICE)			(HACAM hACam, TCHAR szDeviceName[MAX_CAMERA_NUM][256]);
typedef BOOL		(*AT_GETDEVICENAME)			(HACAM hACam, LONG index, LPTSTR szDeviceName, LONG nSize);

//ver 1010
typedef float		(*AT_GETINTERNALTEMPERATURE)(HACAM hACam, LPBOOL Error);

typedef BOOL		(*AT_SETOFFSETVALUE)		(HACAM hACam, WORD Data, double Temperature);
typedef double		(*AT_GETOFFSETVALUE)		(HACAM hACam, WORD Data, LPBOOL Error);

typedef BOOL		(*AT_SETTHERMOTABLE)		(HACAM hACam, WORD Data, double Temperature);
typedef double		(*AT_GETTHERMOTABLE)		(HACAM hACam, WORD Data, LPBOOL Error);


class CArtCamSdk{
private:
	HINSTANCE	m_hArtCamSdk;
	HACAM		m_hACam;


	AT_GETDLLVERSION	m_GetDllVersion;
	AT_GETLASTERROR		m_GetLastError;

	AT_INITIALIZE		m_Initialize;
	AT_RELEASE			m_Release;

	AT_SETWINDOWHANDLE	m_SetWindowHandle;
	AT_SETCAMERATYPE	m_SetCameraType;
	AT_SETCAMERANUMBER	m_SetCameraNumber;
	AT_ISCAMERAENABLE	m_IsCameraEnable;

	AT_SETPARAM			m_SetParam;
	AT_GETPARAM			m_GetParam;

	AT_PREVIEW			m_Preview;
	AT_CLOSE			m_Close;
	AT_START			m_Start;
	AT_PAUSE			m_Pause;

	AT_ONESHOT			m_OneShot;
	AT_SAVE				m_Save;
	AT_REC				m_Rec;

	AT_GETNEWID			m_GetNewID;
	AT_GETRAW			m_GetRaw;
	AT_GETTEMPERATURE	m_GetTemperature;
	AT_GETCOLOR			m_GetColor;

	AT_NUC				m_Nuc;
	AT_SETTABLE			m_SetTable;

	AT_SETIPADDRESS				m_SetIpAddress;


	AT_SETDEVICENUMBER			m_SetDeviceNumber;
	AT_ENUMDEVICE				m_EnumDevice;
	AT_GETDEVICENAME			m_GetDeviceName;
	AT_CONNECTTOCAMERA			m_ConnectToCamera;

	AT_GETINTERNALTEMPERATURE	m_GetInternalTemperature;

	AT_SETOFFSETVALUE	m_SetOffsetValue;
	AT_GETOFFSETVALUE	m_GetOffsetValue;

	AT_SETTHERMOTABLE	m_SetThermoTable;
	AT_GETTHERMOTABLE	m_GetThermoTable;

public:

	DWORD		GetDllVersion	(void);
	LONG		GetLastError	(void);

	BOOL		Initialize		(HWND hWnd);
	BOOL		Release			(void);

	BOOL		SetWindowHandle	(HWND hWnd);
	BOOL		SetCameraType	(ARTCAM_CAMERATYPE Type);
	BOOL		SetCameraNumber	(LONG Number);
	BOOL		IsCameraEnable	(LONG index, LPTSTR szDeviceName, LONG nSize);

	BOOL		SetParam		(ARTCAM_PARAM Param, LONG Value);
	LONG		GetParam		(ARTCAM_PARAM Param);

	BOOL		Preview			(void);
	BOOL		Close			(void);
	BOOL		Start			(void);
	BOOL		Pause			(void);

	DWORD		OneShot			(void);
	BOOL		Save			(LPCTSTR szFileName, FILESAVETYPE FileType);
	BOOL		Rec				(LPCTSTR szAviName, RECMODE Mode);

	DWORD		GetNewID		(void);
	BOOL		GetRaw			(DWORD ID, LPBYTE pData, DWORD Size);
	BOOL		GetTemperature	(DWORD ID, float* pData, DWORD Size);
	BOOL		GetColor		(DWORD ID, LPBYTE pImage, DWORD Size);

	BOOL		Nuc				(void);
	BOOL		SetTable		(LPCTSTR szFileName);

	BOOL		SetIpAddress			(BYTE Add1, BYTE Add2, BYTE Add3, BYTE Add4);

	BOOL		SetDeviceNumber			(LONG Number);
	LONG		EnumDevice				(TCHAR szDeviceName[MAX_CAMERA_NUM][256]);
	BOOL		GetDeviceName			(LONG index, LPTSTR szDeviceName, LONG nSize);
	BOOL		ConnectToCamera			(BYTE Address[6]);

	float		GetInternalTemperature	(LPBOOL Error);

	BOOL		SetOffsetValue	(WORD Data, double Temperature);
	double		GetOffsetValue	(WORD Data, LPBOOL Error=NULL);

	BOOL		SetThermoTable	(WORD Data, double Temperature);
	double		GetThermoTable	(WORD Data, LPBOOL Error=NULL);


public:
	CArtCamSdk(){
		NullSet();
	}

	~CArtCamSdk(){
		FreeLibrary();
	}

	BOOL	LoadLibrary	(ARTCAM_CAMERATYPE DllType);
	BOOL	LoadLibrary	(LPTSTR szDllName);
	void	FreeLibrary	(void);

	BOOL	IsInit		(void) { return m_hArtCamSdk ? TRUE : FALSE; }


public:
	LONG Width(void)
	{
		return GetParam(PARAM_WIDTH);
	}

	LONG Height(void)
	{
		return GetParam(PARAM_HEIGHT);
	}

	BOOL NucAuto(BOOL AutoMode)
	{
		return SetParam(PARAM_NUC_AUTO, AutoMode);
	}

	BOOL IsNucAuto(void)
	{
		return GetParam(PARAM_NUC_AUTO) ? TRUE : FALSE;
	}

	BOOL SetRange(RANGE Range)
	{
		return SetParam(PARAM_RANGE, Range);
	}

	RANGE GetRange(void)
	{
		return (RANGE)GetParam(PARAM_RANGE);
	}


protected:
	void	NullSet		(void);
};

inline void CArtCamSdk::NullSet(void)
{
	m_hArtCamSdk			= NULL;
	m_hACam					= NULL;

	m_GetDllVersion			= NULL;
	m_GetLastError			= NULL;

	m_Initialize			= NULL;
	m_Release				= NULL;

	m_SetWindowHandle		= NULL;
	m_SetCameraType			= NULL;
	m_SetCameraNumber		= NULL;
	m_IsCameraEnable		= NULL;

	m_SetParam				= NULL;
	m_GetParam				= NULL;

	m_Preview				= NULL;
	m_Close					= NULL;
	m_Start					= NULL;
	m_Pause					= NULL;

	m_OneShot				= NULL;
	m_Save					= NULL;
	m_Rec					= NULL;

	m_GetNewID				= NULL;
	m_GetRaw				= NULL;
	m_GetTemperature		= NULL;
	m_GetColor				= NULL;

	m_Nuc					= NULL;
	m_SetTable				= NULL;

	m_SetIpAddress			= NULL;

	m_SetDeviceNumber		= NULL;
	m_EnumDevice			= NULL;
	m_GetDeviceName			= NULL;
	m_ConnectToCamera		= NULL;
	
	m_GetInternalTemperature= NULL;

	m_SetOffsetValue		= NULL;
	m_GetOffsetValue		= NULL;
	m_SetThermoTable		= NULL;
	m_GetThermoTable		= NULL;

}


inline BOOL CArtCamSdk::LoadLibrary(ARTCAM_CAMERATYPE DllType)
{
	switch(DllType){
		case ARTCAM_CAMERATYPE_320:		return LoadLibrary(_T("ArtCamSdk_Thermo320.dll"));
		case ARTCAM_CAMERATYPE_640:		return LoadLibrary(_T("ArtCamSdk_Thermo640.dll"));
		case ARTCAM_CAMERATYPE_320LAN:	return LoadLibrary(_T("ArtCamSdk_Thermo320Lan.dll"));
		case ARTCAM_CAMERATYPE_640LAN:	return LoadLibrary(_T("ArtCamSdk_Thermo640Lan.dll"));
		case ARTCAM_CAMERATYPE_320GIGE:	return LoadLibrary(_T("ArtCamSdk_Thermo320GigE.dll"));
		case ARTCAM_CAMERATYPE_640GIGE:	return LoadLibrary(_T("ArtCamSdk_Thermo640GigE.dll"));
		case ARTCAM_CAMERATYPE_320WOM:	return LoadLibrary(_T("ArtCamSdk_Thermo320_WOM.dll"));
		case ARTCAM_CAMERATYPE_640WOM:	return LoadLibrary(_T("ArtCamSdk_Thermo640_WOM.dll"));
		case ARTCAM_CAMERATYPE_320LAN2:	return LoadLibrary(_T("ArtCamSdk_Thermo320Lan2.dll"));
		case ARTCAM_CAMERATYPE_640LAN2:	return LoadLibrary(_T("ArtCamSdk_Thermo640Lan2.dll"));
		case ARTCAM_CAMERATYPE_320LAN3:	return LoadLibrary(_T("ArtCamSdk_Thermo320Lan3.dll"));
		case ARTCAM_CAMERATYPE_640LAN3:	return LoadLibrary(_T("ArtCamSdk_Thermo640Lan3.dll"));
		case ARTCAM_CAMERATYPE_640P:   	return LoadLibrary(_T("ArtCamSdk_Thermo640P.dll"));
		case ARTCAM_CAMERATYPE_384P:   	return LoadLibrary(_T("ArtCamSdk_Thermo384P.dll"));
	}

	return FALSE;
}


inline BOOL CArtCamSdk::LoadLibrary(LPTSTR szDllName)
{
	if(m_hArtCamSdk)
		return TRUE;


	m_hArtCamSdk = ::LoadLibrary(szDllName);

	if(NULL == m_hArtCamSdk){
		return FALSE;
	}


	// Version check of DLL
	m_GetDllVersion				= (AT_GETDLLVERSION)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetDllVersion");
	if(!m_GetDllVersion){
		FreeLibrary();
		return FALSE;
	}


	// Get version
	WORD Version = LOWORD(GetDllVersion());
	WORD DllType = HIWORD(GetDllVersion());

	// Normal function
	if(1000 <= Version){
		m_GetLastError			= (AT_GETLASTERROR)			::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetLastError");
		m_Initialize			= (AT_INITIALIZE)			::GetProcAddress(m_hArtCamSdk, "ArtThermo_Initialize");
		m_Release				= (AT_RELEASE)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_Release");
		m_SetWindowHandle		= (AT_SETWINDOWHANDLE)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_SetWindowHandle");
		m_SetCameraType			= (AT_SETCAMERATYPE)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_SetCameraType");
		m_SetCameraNumber		= (AT_SETCAMERANUMBER)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_SetCameraNumber");
		m_SetParam				= (AT_SETPARAM)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_SetParam");
		m_GetParam				= (AT_GETPARAM)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetParam");
		m_Preview				= (AT_PREVIEW)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_Preview");
		m_Close					= (AT_CLOSE)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_Close");
		m_Start					= (AT_START)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_Start");
		m_Pause					= (AT_PAUSE)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_Pause");
		m_OneShot				= (AT_ONESHOT)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_OneShot");
		m_GetNewID				= (AT_GETNEWID)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetNewID");
		m_GetRaw				= (AT_GETRAW)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetRaw");
		m_GetTemperature		= (AT_GETTEMPERATURE)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetTemperature");
		m_GetColor				= (AT_GETCOLOR)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetColor");
		m_Nuc					= (AT_NUC)					::GetProcAddress(m_hArtCamSdk, "ArtThermo_Nuc");

#ifdef UNICODE
		m_IsCameraEnable		= (AT_ISCAMERAENABLE)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_IsCameraEnableW");
		m_Save					= (AT_SAVE)					::GetProcAddress(m_hArtCamSdk, "ArtThermo_SaveW");
		m_Rec					= (AT_REC)					::GetProcAddress(m_hArtCamSdk, "ArtThermo_RecW");
		m_SetTable				= (AT_SETTABLE)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_SetTableW");
#else
		m_IsCameraEnable		= (AT_ISCAMERAENABLE)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_IsCameraEnableA");
		m_Save					= (AT_SAVE)					::GetProcAddress(m_hArtCamSdk, "ArtThermo_SaveA");
		m_Rec					= (AT_REC)					::GetProcAddress(m_hArtCamSdk, "ArtThermo_RecA");
		m_SetTable				= (AT_SETTABLE)				::GetProcAddress(m_hArtCamSdk, "ArtThermo_SetTableA");
#endif

		if(	!m_GetLastError || !m_Initialize || !m_Release || !m_SetWindowHandle || !m_SetCameraType || !m_SetCameraNumber || !m_IsCameraEnable || 
			!m_SetParam || !m_GetParam || !m_Preview || !m_Close || !m_Start || !m_Pause || !m_OneShot || !m_Save || !m_Rec || 
			!m_GetNewID || !m_GetRaw || !m_GetTemperature || !m_GetColor || !m_Nuc || !m_SetTable)
		{
			FreeLibrary();
			return FALSE;
		}


		//LAN-Type
		if(ARTCAM_CAMERATYPE_320LAN == DllType || ARTCAM_CAMERATYPE_640LAN == DllType ||
		   ARTCAM_CAMERATYPE_320LAN2== DllType || ARTCAM_CAMERATYPE_640LAN2== DllType ||
		   ARTCAM_CAMERATYPE_320LAN3== DllType || ARTCAM_CAMERATYPE_640LAN3== DllType){
			m_SetIpAddress			= (AT_SETIPADDRESS)			::GetProcAddress(m_hArtCamSdk, "ArtThermo_SetIpAddress");
			if(!m_SetIpAddress)
			{
				FreeLibrary();
				return FALSE;
			}
	}

		//GigE-Type
		if(ARTCAM_CAMERATYPE_320GIGE == DllType || ARTCAM_CAMERATYPE_640GIGE == DllType){
			m_SetDeviceNumber		= (AT_SETDEVICENUMBER)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_SetDeviceNumber");
			m_ConnectToCamera		= (AT_CONNECTTOCAMERA)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_ConnectToCamera");
#ifdef UNICODE
			m_EnumDevice			= (AT_ENUMDEVICE)			::GetProcAddress(m_hArtCamSdk, "ArtThermo_EnumDeviceW");
			m_GetDeviceName			= (AT_GETDEVICENAME)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetDeviceNameW");
#else
			m_EnumDevice			= (AT_ENUMDEVICE)			::GetProcAddress(m_hArtCamSdk, "ArtThermo_EnumDeviceA");
			m_GetDeviceName			= (AT_GETDEVICENAME)		::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetDeviceNameA");
#endif
			if(!m_SetDeviceNumber || !m_EnumDevice || !m_GetDeviceName || !m_ConnectToCamera)
			{
				FreeLibrary();
				return FALSE;
			}
		}
	}

	if(1010 <= Version){
		m_GetInternalTemperature	= (AT_GETINTERNALTEMPERATURE)::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetInternalTemperature");
		if(	!m_GetInternalTemperature)
		{
			FreeLibrary();
			return FALSE;
		}

	}

	if(1100 <= Version){
		if(ARTCAM_CAMERATYPE_640P == DllType || ARTCAM_CAMERATYPE_384P == DllType){
			m_SetOffsetValue			= (AT_SETOFFSETVALUE)			::GetProcAddress(m_hArtCamSdk, "ArtThermo_SetOffsetValue");
			m_GetOffsetValue			= (AT_GETOFFSETVALUE)			::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetOffsetValue");
			m_SetThermoTable			= (AT_SETTHERMOTABLE)			::GetProcAddress(m_hArtCamSdk, "ArtThermo_SetThermoTable");
			m_GetThermoTable			= (AT_GETTHERMOTABLE)			::GetProcAddress(m_hArtCamSdk, "ArtThermo_GetThermoTable");
			if(!m_SetOffsetValue || !m_GetOffsetValue || !m_SetThermoTable || !m_GetThermoTable)
			{
				FreeLibrary();
				return FALSE;
			}
		}
	}

	return TRUE;
}

inline void CArtCamSdk::FreeLibrary()
{
	if(m_hACam && m_Release)
		m_Release(m_hACam);

	if(m_hArtCamSdk)
		::FreeLibrary(m_hArtCamSdk);

	NullSet();
}

inline DWORD CArtCamSdk::GetDllVersion(void)
{
	if(m_GetDllVersion){
		return m_GetDllVersion();
	}

	return FALSE;
}


inline LONG CArtCamSdk::GetLastError(void)
{
	if(m_GetLastError){
		return m_GetLastError(m_hACam);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::Initialize(HWND hWnd)
{
	if(m_Initialize){
		return m_hACam = m_Initialize(hWnd);
	}

	return FALSE;

}


inline BOOL CArtCamSdk::Release(void)
{
	if(m_Release){
		return m_Release(m_hACam);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::SetWindowHandle(HWND hWnd)
{
	if(m_SetWindowHandle){
		return m_SetWindowHandle(m_hACam, hWnd);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::SetCameraType(ARTCAM_CAMERATYPE Type)
{
	if(m_SetCameraType){
		return m_SetCameraType(m_hACam, Type);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::SetCameraNumber(LONG Number)
{
	if(m_SetCameraNumber){
		return m_SetCameraNumber(m_hACam, Number);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::IsCameraEnable(LONG index, LPTSTR szDeviceName, LONG nSize)
{
	if(m_IsCameraEnable){
		return m_IsCameraEnable(m_hACam, index, szDeviceName, nSize);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::SetParam(ARTCAM_PARAM Param, LONG Value)
{
	if(m_SetParam){
		return m_SetParam(m_hACam, Param, Value);
	}

	return FALSE;
}


inline LONG CArtCamSdk::GetParam(ARTCAM_PARAM Param)
{
	if(m_GetParam){
		return m_GetParam(m_hACam, Param);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::Preview(void)
{
	if(m_Preview){
		return m_Preview(m_hACam);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::Close(void){
	if(m_Close){
		return m_Close(m_hACam);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::Start(void)
{
	if(m_Start){
		return m_Start(m_hACam);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::Pause(void)
{
	if(m_Pause){
		return m_Pause(m_hACam);
	}

	return FALSE;
}


inline DWORD CArtCamSdk::OneShot(void)
{
	if(m_OneShot){
		return m_OneShot(m_hACam);
	}

	return 0;
}


inline BOOL CArtCamSdk::Save(LPCTSTR szFileName, FILESAVETYPE FileType)
{
	if(m_Save){
		return m_Save(m_hACam, szFileName, FileType);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::Rec(LPCTSTR szAviName, RECMODE Mode)
{
	if(m_Rec){
		return m_Rec(m_hACam, szAviName, Mode);
	}

	return FALSE;
}


inline DWORD CArtCamSdk::GetNewID(void)
{
	if(m_GetNewID){
		return m_GetNewID(m_hACam);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::GetRaw(DWORD ID, LPBYTE pData, DWORD Size)
{
	if(m_GetRaw){
		return m_GetRaw(m_hACam, ID, pData, Size);
	}
	
	return FALSE;
}


inline BOOL CArtCamSdk::GetTemperature(DWORD ID, float* pData, DWORD Size)
{
	if(m_GetTemperature){
		return m_GetTemperature(m_hACam, ID, pData, Size);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::GetColor(DWORD ID, LPBYTE pImage, DWORD Size)
{
	if(m_GetColor){
		return m_GetColor(m_hACam, ID, pImage, Size);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::Nuc(void)
{
	if(m_Nuc){
		return m_Nuc(m_hACam);
	}

	return FALSE;
}


inline BOOL CArtCamSdk::SetTable(LPCTSTR szFileName)
{
	if(m_SetTable){
		return m_SetTable(m_hACam, szFileName);
	}

	return FALSE;
}

inline BOOL CArtCamSdk::SetIpAddress(BYTE Add1, BYTE Add2, BYTE Add3, BYTE Add4)
{
	if(m_SetIpAddress){
		return m_SetIpAddress(m_hACam, Add1, Add2, Add3, Add4);
	}

	return FALSE;
}

inline BOOL CArtCamSdk::SetDeviceNumber(LONG Number)
{
	if(m_SetDeviceNumber){
		return m_SetDeviceNumber(m_hACam, Number);
	}
	return FALSE;
}

inline LONG CArtCamSdk::EnumDevice(TCHAR szDeviceName[MAX_CAMERA_NUM][256])
{
	if(m_EnumDevice){
		return m_EnumDevice(m_hACam, szDeviceName);
	}
	return 0;
}

inline BOOL CArtCamSdk::GetDeviceName(LONG index, LPTSTR szDeviceName, LONG nSize)
{
	if(m_GetDeviceName){
		return m_GetDeviceName(m_hACam, index, szDeviceName, nSize);
	}
	return FALSE;
}

inline BOOL CArtCamSdk::ConnectToCamera(BYTE Address[6])
{
	if(m_ConnectToCamera){
		return m_ConnectToCamera(m_hACam, Address);
	}
	return FALSE;
}


inline BOOL CArtCamSdk::SetOffsetValue(WORD Data, double Temperature)
{
	if(m_SetOffsetValue){
		return m_SetOffsetValue(m_hACam, Data, Temperature);
	}
	return FALSE;
}

inline float CArtCamSdk::GetInternalTemperature(LPBOOL Error)
{
	if(m_GetInternalTemperature){
		return m_GetInternalTemperature(m_hACam, Error);
	}
	return FALSE;
}

inline double CArtCamSdk::GetOffsetValue(WORD Data, LPBOOL Error)
{
	if(m_GetOffsetValue){
		return m_GetOffsetValue(m_hACam, Data, Error);
	}
	return 0.0;
}

inline BOOL CArtCamSdk::SetThermoTable(WORD Data, double Temperature)
{
	if(m_SetThermoTable){
		return m_SetThermoTable(m_hACam, Data, Temperature);
	}
	return FALSE;
}

inline double CArtCamSdk::GetThermoTable(WORD Data, LPBOOL Error)
{
	if(m_GetThermoTable){
		return m_GetThermoTable(m_hACam, Data, Error);
	}
	return 0.0;
}

#endif	// #ifndef _CARTCAMSDK_
