// ParamSetupDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Sample_MFC.h"
#include "ParamSetupDlg.h"


// CParamSetupDlg dialog

IMPLEMENT_DYNAMIC(CParamSetupDlg, CDialog)
CParamSetupDlg::CParamSetupDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CParamSetupDlg::IDD, pParent)
{
	m_pArtCam	= NULL;
	m_pParam	= NULL;
}

CParamSetupDlg::~CParamSetupDlg()
{
}

void CParamSetupDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	DDX_Control(pDX, IDC_SETRANGE, m_ComboRange); 
}


BEGIN_MESSAGE_MAP(CParamSetupDlg, CDialog)
	ON_BN_CLICKED(IDB_NUC,			OnBnClickedNuc)
	ON_BN_CLICKED(IDC_NUCAUTO,		OnBnClickedAutonuc)
	ON_CBN_SELCHANGE(IDC_SETRANGE,	OnCbnSelchangeSetrange)
	ON_BN_CLICKED(IDC_DEFAULT,		OnBnClickedDefault)
	ON_BN_CLICKED(IDB_APPLAY,		OnBnClickedApply)
	ON_BN_CLICKED(IDC_SHOW_MIRROR,	OnBnClickedShowMirror)
	ON_BN_CLICKED(IDC_DATA_MIRROR,	OnBnClickedDataMirror)
END_MESSAGE_MAP()


// CParamSetupDlg message handlers


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Dialog format process
// 
//
BOOL CParamSetupDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Get current parameter
	if(NULL != m_pArtCam && NULL != m_pParam){
		SetDlgItemInt(IDE_WAITTIME,		m_pParam->WaitTime	);				// Frame rate
		SetDlgItemInt(IDE_RINGCOUNT,	m_pParam->BufferSize);				// Buffer size
		SetDlgItemInt(IDE_COLOR_AUTO,	m_pParam->ColorAuto	);				// Number of adjustment times of pseudo color
		SetDlgItemInt(IDE_COLOR_MIN,	m_pParam->ColorMin	);				// Minimum value of pseudo color
		SetDlgItemInt(IDE_COLOR_MAX,	m_pParam->ColorMax	);				// Maximum value of pseudo color
		SetDlgItemInt(IDE_WAITNUC,		m_pParam->WaitNuc	);				// NUC pausing timeimsec)

		// Currently, you cannot change inside buffer
		GetDlgItem(IDE_RINGCOUNT)->EnableWindow(FALSE);
				
		CheckDlgButton(IDC_NUCAUTO,		m_pParam->AutoNuc		? BST_CHECKED : BST_UNCHECKED);		// Auto NUC
		CheckDlgButton(IDC_SHOW_MIRROR, m_pParam->ShowMirrorV	? BST_CHECKED : BST_UNCHECKED);		// Upside down received data
		CheckDlgButton(IDC_DATA_MIRROR, m_pParam->DataMirrorV	? BST_CHECKED : BST_UNCHECKED);		// Upside down saved data

		// Temperature range
		m_ComboRange.InsertString(0, _T("Auto"));
		m_ComboRange.InsertString(1, _T("LO"));
		m_ComboRange.InsertString(2, _T("HI"));
		m_ComboRange.SetCurSel((int)m_pParam->Range);

		// Data range
		if(0 != m_pParam->ColorAuto){
			GetDlgItem(IDE_COLOR_MIN)->EnableWindow(FALSE);
			GetDlgItem(IDE_COLOR_MAX)->EnableWindow(FALSE);
		}
	}

	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	NUC
// 
//
void CParamSetupDlg::OnBnClickedNuc()
{
	// NUC only 1 time
	if(NULL != m_pArtCam){
		m_pArtCam->Nuc();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Switch auto NUC mode
// 
//
void CParamSetupDlg::OnBnClickedAutonuc()
{
	// Effect auto NUC
	// In this case, NUC works when it's needed
	if(NULL != m_pArtCam && NULL != m_pParam){
		m_pParam->AutoNuc = IsDlgButtonChecked(IDC_NUCAUTO) ? TRUE : FALSE;
		m_pArtCam->NucAuto(m_pParam->AutoNuc);
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Switch temperature range (Auto/High/Low)
// 
//
void CParamSetupDlg::OnCbnSelchangeSetrange()
{
	// Change temperature range
	if(NULL != m_pArtCam && NULL != m_pParam){
		switch(m_ComboRange.GetCurSel()){
			case 0:	m_pArtCam->SetRange((m_pParam->Range = RANGE_AUTO));	break;
			case 1:	m_pArtCam->SetRange((m_pParam->Range = RANGE_LO  ));	break;
			case 2:	m_pArtCam->SetRange((m_pParam->Range = RANGE_HI  ));	break;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Default setting
// 
//
void CParamSetupDlg::OnBnClickedDefault()
{
	// Back to default setting
	if(NULL != m_pArtCam && NULL != m_pParam){
		SetDlgItemInt(IDE_WAITTIME,		(m_pParam->WaitTime		=    10));
		SetDlgItemInt(IDE_COLOR_AUTO,	(m_pParam->ColorAuto	=    -1));
		SetDlgItemInt(IDE_COLOR_MIN,	(m_pParam->ColorMin		=  1000));
		SetDlgItemInt(IDE_COLOR_MAX,	(m_pParam->ColorMax		=  3000)); 
		SetDlgItemInt(IDE_WAITNUC,		(m_pParam->WaitNuc		=  1000));
		CheckDlgButton(IDC_NUCAUTO,		(m_pParam->AutoNuc		=  TRUE));
		CheckDlgButton(IDC_SHOW_MIRROR, (m_pParam->ShowMirrorV  = FALSE));
		CheckDlgButton(IDC_DATA_MIRROR, (m_pParam->DataMirrorV	= FALSE));
		
		m_pParam->Range = RANGE_AUTO;
		m_ComboRange.SetCurSel((LONG)m_pParam->Range);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Reflect setting
// 
//
void CParamSetupDlg::OnBnClickedApply()
{
	if(NULL != m_pArtCam && NULL != m_pParam){
		m_pArtCam->SetParam(PARAM_WAITTIME,		(m_pParam->WaitTime		= (LONG)GetDlgItemInt(IDE_WAITTIME  )));
		m_pArtCam->SetParam(PARAM_WAITNUC,		(m_pParam->WaitNuc		= (LONG)GetDlgItemInt(IDE_WAITNUC   )));
		m_pArtCam->SetParam(PARAM_COLOR_AUTO,	(m_pParam->ColorAuto	= (LONG)GetDlgItemInt(IDE_COLOR_AUTO)));

		/*****************************************************************************/
		// Receive/set range of indicating data
		// Set a value of 14bit integral number(0`16383) that is camera's output data
		// Basically you can get fit pseudo color
		// by inputting output maximum and minimum data
		// This setting is same factor when to indicate monochrome
		/*****************************************************************************/

		// Manual setting
		if(0 == m_pParam->ColorAuto){
			m_pArtCam->SetParam(PARAM_COLOR_MIN, (m_pParam->ColorMin = GetDlgItemInt(IDE_COLOR_MIN)));
			m_pArtCam->SetParam(PARAM_COLOR_MAX, (m_pParam->ColorMax = GetDlgItemInt(IDE_COLOR_MAX)));
			GetDlgItem(IDE_COLOR_MIN)->EnableWindow(TRUE);
			GetDlgItem(IDE_COLOR_MAX)->EnableWindow(TRUE);
		// Auto correction only between number of set frames
		}else if(0 < m_pParam->ColorAuto){
			// Sleep during auto correction(Does not indicate if correction is done)
			Sleep(100);
			SetDlgItemInt(IDE_COLOR_MIN, (m_pParam->ColorMin = m_pArtCam->GetParam(PARAM_COLOR_MIN)));
			SetDlgItemInt(IDE_COLOR_MAX, (m_pParam->ColorMax = m_pArtCam->GetParam(PARAM_COLOR_MAX)));
			GetDlgItem(IDE_COLOR_MIN)->EnableWindow(FALSE);
			GetDlgItem(IDE_COLOR_MAX)->EnableWindow(FALSE);
		// Auto carrection to all frames
		}else{
			GetDlgItem(IDE_COLOR_MIN)->EnableWindow(FALSE);
			GetDlgItem(IDE_COLOR_MAX)->EnableWindow(FALSE);
		}
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Vertical mirror reverse setting
//  
//
void CParamSetupDlg::OnBnClickedShowMirror()
{
	// This setting is for received data by Get*** function to do mirror reversing
	if(NULL != m_pArtCam && NULL != m_pParam){
		m_pParam->ShowMirrorV = IsDlgButtonChecked(IDC_SHOW_MIRROR) ? TRUE : FALSE;
		m_pArtCam->SetParam(PARAM_SHOW_MIRROR, m_pParam->ShowMirrorV);
	}
}

void CParamSetupDlg::OnBnClickedDataMirror()
{
	// This setting is for received data by Save function to do mirror reversing
	// * Does not change on a monitor
	if(NULL != m_pArtCam && NULL != m_pParam){
		m_pParam->DataMirrorV = IsDlgButtonChecked(IDC_DATA_MIRROR) ? TRUE : FALSE;
		m_pArtCam->SetParam(PARAM_DATA_MIRROR, m_pParam->DataMirrorV);
	}
}
