// Sample_MFCView.cpp : implementation of the CSample_MFCView class
//

#include "stdafx.h"
#include "Sample_MFC.h"
#include "Sample_MFCView.h"
#include "Sample_MFCDoc.h"
#include "SaveFileDlg.h"
#include "SaveRecDlg.h"
#include "ParamSetupDlg.h"
#include "ReplayDlg.h"
#include "OffsetDlg.h"
#include "MainFrm.h"
#include "IpSetDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif

static const int ARTCAM_CAMERATYPE_NUM[DLL_COUNT] = { 1,2,3,4,5,6,7,8,9,10,11,12,20,21 };


#define SAFE_DELETE(x)  { if(x) delete[] x; x=NULL; }


//Text for indicating error
const TCHAR SZERRORLIST[][128] = {
	_T("Normal"),
	_T("Not formatted yet"),
	_T("Error invalid device"),
	_T("Fail image scanning"),
	_T("Lost device"),
	_T("Abnormal argument"),
	_T("Not enough memory for tranferring"),
	_T("Fail to create thread for image scanning"),
	_T("Error camera (device) setting"),
	_T("Cannot find specified file"),
	_T("THis function is not supported"),
	_T("Abnormal Termination"),
};

// CSample_MFCView

IMPLEMENT_DYNCREATE(CSample_MFCView, CView)

BEGIN_MESSAGE_MAP(CSample_MFCView, CView)

	ON_WM_CREATE()									// Mapping to start application
	ON_WM_DESTROY()									// Mapping to finish application
	ON_WM_PAINT()									// Use instead of OnDraw

	ON_MESSAGE(WM_GRAPHPAINT,	OnGraphPaint)		// Call when image is updated
	ON_MESSAGE(WM_ERROR,		OnError)			// Call when error is occured

	// File menu
	ON_COMMAND(IDM_SAVEFILE,	FileSave)			// Save an image dialog
	ON_COMMAND(IDM_RECFILE,		RecSave)			// Recording dialog
	ON_COMMAND(IDM_READ_TABLE,	OnReadTableFile)	// Read temperature table
	ON_COMMAND(IDM_READ_DATA,	OnReadData)			// Read data file

	// Indication menu
	ON_COMMAND(IDM_SHOW_PREVIEW,	OnShowPreview)	// Preview
	ON_COMMAND(IDM_SHOW_SNAPSHOT,	OnShowSnapShot)	// Snapshot
	ON_COMMAND(IDM_SHOW_STOP,		ShowStop)		// StopEPlay

	// Set up menu
	ON_COMMAND(IDM_PARAM_SETUP, OpenParamDlg)		// Set up camra dialog
	ON_COMMAND(IDM_DISPLAY_X1,	OnDisplayX1)		// Set up magnification	100%
	ON_COMMAND(IDM_DISPLAY_X2,	OnDisplayX2)		// Set up magnification	200%
	ON_COMMAND(IDM_DISPLAY_X3,	OnDisplayX3)		// Set up magnification	300%

	// Device menu
	ON_COMMAND(IDM_DEVICE01, OnDevice01)			// Device menu 1
	ON_COMMAND(IDM_DEVICE02, OnDevice02)			// Device menu 2
	ON_COMMAND(IDM_DEVICE03, OnDevice03)			// Device menu 3
	ON_COMMAND(IDM_DEVICE04, OnDevice04)			// Device menu 4
	ON_COMMAND(IDM_DEVICE05, OnDevice05)			// Device menu 5
	ON_COMMAND(IDM_DEVICE06, OnDevice06)			// Device menu 6
	ON_COMMAND(IDM_DEVICE07, OnDevice07)			// Device menu 7
	ON_COMMAND(IDM_DEVICE08, OnDevice08)			// Device menu 8


	// Switch indication
	ON_COMMAND(IDM_COLORMONO,	OnColorMono)

	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(IDM_SHOW_RAW, OnShowRaw)
	ON_COMMAND(IDM_SHOW_TEMP, OnShowTemp)
	ON_COMMAND(IDM_SETIP, OnSetip)
	ON_COMMAND(ID_DLL_320USB, OnDll320usb)
	ON_COMMAND(ID_DLL_640USB, OnDll640usb)
	ON_COMMAND(ID_DLL_320LAN, OnDll320lan)
	ON_COMMAND(ID_DLL_640LAN, OnDll640lan)
	ON_COMMAND(ID_DLL_320GIGE, OnDll320gige)
	ON_COMMAND(ID_DLL_640GIGE, OnDll640gige)
	ON_COMMAND(ID_DLL_320WOM, OnDll320wom)
	ON_COMMAND(ID_DLL_640WOM, OnDll640wom)
	ON_COMMAND(ID_DLL_320LAN2,OnDll320lan2)
	ON_COMMAND(ID_DLL_640LAN2,OnDll640lan2)
	ON_COMMAND(ID_DLL_320LAN3,OnDll320lan3)
	ON_COMMAND(ID_DLL_640LAN3,OnDll640lan3)
	ON_COMMAND(ID_DLL_640P,   OnDll640p)
	ON_COMMAND(ID_DLL_384P,   OnDll384p)
	ON_WM_TIMER()
END_MESSAGE_MAP()



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Process of regulation
//
//
BOOL CSample_MFCView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CView::PreCreateWindow(cs);
}

void CSample_MFCView::OnDraw(CDC* pDC)
{
	CSample_MFCDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
}

#ifdef _DEBUG
void CSample_MFCView::AssertValid() const
{
	CView::AssertValid();
}

void CSample_MFCView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CSample_MFCDoc* CSample_MFCView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CSample_MFCDoc)));
	return (CSample_MFCDoc*)m_pDocument;
}
#endif //_DEBUG



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	ConstructorEDestructor
//
//
CSample_MFCView::CSample_MFCView()
{
	theApp.m_pView	= this;		// Status bar for updating

	m_bShowStop		= FALSE;	// Preview mode
	m_iDisplayOp	= -1;		// Indicating mode
	m_SelectDevice	= -1;		// Format choosen device
	m_iDeviceCount	= 0;		// Device number
	m_iStretchSize	= 1;		// Magnification rate
	m_Timer			= 0;		


	// Default dataEcolor mode
	m_GetFlg.ViewColor	= TRUE;	
	m_GetFlg.GetColor	= TRUE;
	m_GetFlg.GetRaw		= TRUE;
	m_GetFlg.GetTemp	= FALSE;

	// Format Param data at 0
	ZeroMemory(&m_Param, sizeof(THERMOPARAM));
	m_CameraType = -1;
	for(int i=0; i<DLL_COUNT; i++)
		m_DllEnable[i] = FALSE;

	ZeroMemory(&m_OffsetValue, sizeof(m_OffsetValue));

	m_Initialized = FALSE;

}

CSample_MFCView::~CSample_MFCView()
{

}



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Start
//
//
int CSample_MFCView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1) {
		return -1;
	}

	CMenu* pMenu = GetParent()->GetMenu();
	CMenu* pDllMenu = pMenu->GetSubMenu(4);

	CArtCamSdk	Cam;
	for(int i=0; i<DLL_COUNT; i++){
		if(Cam.LoadLibrary((ARTCAM_CAMERATYPE)ARTCAM_CAMERATYPE_NUM[i])){
			m_DllEnable[i] = TRUE;
			Cam.FreeLibrary();
		}
	}

	return 1;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Finish
//
//
void CSample_MFCView::OnDestroy()
{
	m_ArtCam.Release();
	KillTimer(m_Timer);
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Craeate a menu
// 
//
void CSample_MFCView::InitMenuPopup(CMenu* pPopupMenu)
{
	CMenu* pMenu = GetParent()->GetMenu();
	CMenu* pDeviceMenu0 = pMenu->GetSubMenu(0);  // File menu
	CMenu* pDeviceMenu1 = pMenu->GetSubMenu(1);  // Indication menu
	CMenu* pDeviceMenu2 = pMenu->GetSubMenu(2);  // Set up menu
	CMenu* pDeviceMenu4 = pMenu->GetSubMenu(4);	 // DLL menu
	CMenu* pDeviceMenu5 = pMenu->GetSubMenu(5);  // Device menu

	// Save file menu Show / Hide
	if (m_iDisplayOp == 1 || m_iDisplayOp == 2 && pPopupMenu == pDeviceMenu0) {
		pDeviceMenu0->EnableMenuItem(1, MF_BYPOSITION | MF_ENABLED);
	} else {
		pDeviceMenu0->EnableMenuItem(1, MF_BYPOSITION | MF_GRAYED);
	}

	// Reacord file menu Show / Hide
	if (m_iDisplayOp == 1 && m_bShowStop == FALSE ) {
		pDeviceMenu0->EnableMenuItem(2, MF_BYPOSITION | MF_ENABLED);
	} else {
		pDeviceMenu0->EnableMenuItem(2, MF_BYPOSITION | MF_GRAYED);
	}


	// Data to indicate to a status bar
	if(m_GetFlg.GetRaw)	pDeviceMenu1->CheckMenuItem(IDM_SHOW_RAW, MF_CHECKED);
	else				pDeviceMenu1->CheckMenuItem(IDM_SHOW_RAW, MF_UNCHECKED);

	if(m_GetFlg.GetTemp)pDeviceMenu1->CheckMenuItem(IDM_SHOW_TEMP, MF_CHECKED);
	else				pDeviceMenu1->CheckMenuItem(IDM_SHOW_TEMP, MF_UNCHECKED);


	// Device Show / Hide
	if(pPopupMenu == pDeviceMenu4){
		int i=0;
		for(; i < DLL_COUNT; i++){
			if(m_DllEnable[i])	pPopupMenu->EnableMenuItem(i, MF_BYPOSITION | MF_ENABLED);
			else				pPopupMenu->EnableMenuItem(i, MF_BYPOSITION | MF_GRAYED);
			pPopupMenu->CheckMenuItem(i, MF_BYPOSITION | MF_UNCHECKED);
		}

		if(-1 != m_CameraType)
			pPopupMenu->CheckMenuItem(m_CameraType-1, MF_BYPOSITION | MF_CHECKED);
	}



	// Device Show / Hide
	if(pPopupMenu == pDeviceMenu5){
		int i=0;
		for(; i < m_iDeviceCount; i++){
			pPopupMenu->EnableMenuItem(i, MF_BYPOSITION | MF_ENABLED);
			pPopupMenu->CheckMenuItem(i, MF_BYPOSITION | MF_UNCHECKED);
		}
		for(; i<10; i++){
			pPopupMenu->EnableMenuItem(i, MF_BYPOSITION | MF_GRAYED);
			pPopupMenu->CheckMenuItem(i, MF_BYPOSITION | MF_UNCHECKED);
		}

		pPopupMenu->CheckMenuItem(m_SelectDevice, MF_BYPOSITION | MF_CHECKED);
	}


}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Switch calor view mode
// 
//
void CSample_MFCView::OnColorMono()
{
	// Switch color view mode
	m_GetFlg.ViewColor = !m_GetFlg.ViewColor;

	// Update menu
	GetParent()->GetMenu()->ModifyMenu(
		IDM_COLORMONO, MF_BYCOMMAND | MF_STRING, IDM_COLORMONO, 
		m_GetFlg.ViewColor ? _T("Monochrome") : _T("Color")
	);

	GetParent()->DrawMenuBar();
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Formatting
//  DLL load, Set up each camera, Device check
//
//  
void CSample_MFCView::OnDll320usb(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_320;		InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll640usb(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_640;		InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll320lan(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_320LAN;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll640lan(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_640LAN;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll320gige(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_320GIGE;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll640gige(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_640GIGE;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll320wom(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_320WOM;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll640wom(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_640WOM;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll320lan2(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_320LAN2;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll640lan2(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_640LAN2;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll320lan3(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_320LAN3;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll640lan3(){	m_CameraType = (LONG)ARTCAM_CAMERATYPE_640LAN3;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll640p()	{	m_CameraType = (LONG)ARTCAM_CAMERATYPE_640P;	InitializeArtCam(m_CameraType);	}
void CSample_MFCView::OnDll384p()	{	m_CameraType = (LONG)ARTCAM_CAMERATYPE_384P;	InitializeArtCam(m_CameraType);	}

void CSample_MFCView::InitializeArtCam(LONG Type)
{
	//Release SDK(Need this function when call SDK several times)
	m_ArtCam.FreeLibrary();
	
	LPTSTR szDll = NULL;
	WORD Version = NULL;

	// DLL load
	if (m_ArtCam.LoadLibrary((ARTCAM_CAMERATYPE)Type)) {
		Version = LOWORD(m_ArtCam.GetDllVersion());	 // Get version information
		TCHAR cMainFrameTitle[256];
		switch(Type){
		case ARTCAM_CAMERATYPE_320		: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 320 USB Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_640		: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 640 USB Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_320LAN	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 320 LAN Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_640LAN	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 640 LAN Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_320GIGE	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 320 GigE Ver.%d"),Version);	break;
		case ARTCAM_CAMERATYPE_640GIGE	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 640 GigE Ver.%d"),Version);	break;
		case ARTCAM_CAMERATYPE_320WOM	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 320 WOM Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_640WOM	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 640 WOM Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_320LAN2	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 320 LAN2 Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_640LAN2	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 640 LAN2 Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_320LAN3	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 320 LAN3 Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_640LAN3	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 640 LAN3 Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_640P  	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 640P     Ver.%d"), Version);	break;
		case ARTCAM_CAMERATYPE_384P  	: _stprintf(cMainFrameTitle , _T("Sample_MFC Thermograph Camera 384P     Ver.%d"), Version);	break;
		}
		GetParent()->SetWindowText(cMainFrameTitle);
	} else {
		GetParent()->SetWindowText(_T("Sample_MFC DLL load Error"));
		return;
	}

	CMenu* pMenu = GetParent()->GetMenu();
	CMenu* pDllMenu = pMenu->GetSubMenu(4);

	for(int i=0; i<12; i++){
		pDllMenu->CheckMenuItem(i, MF_BYPOSITION | MF_UNCHECKED);
	}
	pDllMenu->CheckMenuItem(Type, MF_BYPOSITION | MF_CHECKED);

	// Format SDK
	// Pass handle a window to receive image updating or error message
	m_ArtCam.Initialize(GetSafeHwnd());	

	// Get 1 image and default value in data range is set up
	// Need this function later. eg.GetParam(PARAM_COLOR_MIN)
	if(ARTCAM_CAMERATYPE_320 == Type || ARTCAM_CAMERATYPE_640 == Type || ARTCAM_CAMERATYPE_320WOM == Type || ARTCAM_CAMERATYPE_640WOM == Type){
		if (m_ArtCam.IsInit()) {
			m_ArtCam.OneShot();	
		}
	}


	if(ARTCAM_CAMERATYPE_640P == Type ||ARTCAM_CAMERATYPE_384P == Type){
		if (m_ArtCam.IsInit()) {
			OnReadTableFile();
			m_Initialized = TRUE;
		}
	}

	// Get set up value of initialized
	m_Param.WaitTime	= m_ArtCam.GetParam(PARAM_WAITTIME);	// Get frame rate
	m_Param.BufferSize	= m_ArtCam.GetParam(PARAM_RINGCOUNT);	// Get ring buffer size
	m_Param.ColorAuto	= m_ArtCam.GetParam(PARAM_COLOR_AUTO);	// Get number of adjustment of pseudo color times
	m_Param.ColorMin	= m_ArtCam.GetParam(PARAM_COLOR_MIN);	// Basic set up value of minimum value of pseudo color
	m_Param.ColorMax	= m_ArtCam.GetParam(PARAM_COLOR_MAX);	// Basic set up value of maximum value of pseudo color
	m_Param.WaitNuc		= m_ArtCam.GetParam(PARAM_WAITNUC);		// Get NUC stop time(Ms)
	m_Param.AutoNuc		= m_ArtCam.IsNucAuto();					// Basic set up value of AUTONUC
	m_Param.Range		= m_ArtCam.GetRange();					// Gt temperature range
	m_Param.ShowMirrorV	= m_ArtCam.GetParam(PARAM_SHOW_MIRROR);	// Upside down for view mode
	m_Param.DataMirrorV	= m_ArtCam.GetParam(PARAM_DATA_MIRROR);	// Upside down on a data
	m_Param.AviMaxSize  = m_ArtCam.GetParam(PARAM_AVIMAXSIZE);	// Maximum size of AVI file
	m_Param.RecMode		= m_ArtCam.GetParam(PARAM_RECMODE);		// Recording mode
	m_Param.NowRecording= m_ArtCam.GetParam(PARAM_REC);			// Current recording condition

	// Device check
	int iDeviceCount = 0;

	TCHAR szDef[8][256] = { _T("0"), _T("1"), _T("2"), _T("3"), _T("4"), _T("5"), _T("6"), _T("7")};

	CMenu* pDeviceMenu = pMenu->GetSubMenu(5);

	for(int i=0; i<8; i++){
		TCHAR szTemp[256] = _T("");	

		// Device search
		if (m_ArtCam.GetDeviceName(i, szTemp, 256)) { 
			pDeviceMenu->ModifyMenu(iDeviceCount, MF_BYPOSITION | MF_STRING, IDM_DEVICE01 + iDeviceCount, szTemp);
			iDeviceCount++;
		} else {
			pDeviceMenu->ModifyMenu(i, MF_BYPOSITION | MF_STRING, IDM_DEVICE01 + i, szDef[i]);
		} 
	}

	m_iDeviceCount = iDeviceCount;

	m_Timer = SetTimer(1, 5000, NULL);

	OnDeviceChange(0);
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Change connected device
//  
//
BOOL CSample_MFCView::OnDeviceChange(LONG Number)
{
	m_ArtCam.Close();
	// Chenge device number at this funciton. And device is changed when you view a camera
	m_ArtCam.SetDeviceNumber(Number);
	m_SelectDevice = Number;
	
	CMenu* pMenu = GetParent()->GetMenu();
	CMenu* pDeviceMenu = pMenu->GetSubMenu(5);

	for(int i=0; i<8; i++){
		pDeviceMenu->CheckMenuItem(i, MF_BYPOSITION | MF_UNCHECKED);
	}
	pDeviceMenu->CheckMenuItem(Number, MF_BYPOSITION | MF_CHECKED);
	
	if(m_Initialized){
		OnDisplayX2();
		OnShowPreview();
	}

	return TRUE;
}



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Read temperature table
//  
//
void CSample_MFCView::OnReadTableFile()
{
	TCHAR szFile[MAX_PATH]=_T("");
	if(GetDocument()->GetFileNameDlg(
		GetSafeHwnd(), szFile, 
		_T("Temperature table(*.dat *.att *.cds)\0*.dat;*.att;*.cds\0\0"), TRUE, _T("*.dat *.att *.cds"), _T("")))
	{
		m_ArtCam.SetTable(szFile);
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Secure image receiving area
//
//
BOOL CSample_MFCView::CreateImageBuffer(void)
{
	// Solve a problem about memory management in CDib
	if(!m_CDibMono.Create (NULL, m_ArtCam.Width(), m_ArtCam.Height(),  8)) return FALSE;	// BYTE
	if(!m_CDibRaw.Create  (NULL, m_ArtCam.Width(), m_ArtCam.Height(), 16)) return FALSE;	// WORD
	if(!m_CDibColor.Create(NULL, m_ArtCam.Width(), m_ArtCam.Height(), 24)) return FALSE;	// BYTE[3]
	if(!m_CDibTemp.Create (NULL, m_ArtCam.Width(), m_ArtCam.Height(), 32)) return FALSE;	// float

	return TRUE;
}




//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Preview
// 
//
void CSample_MFCView::OnShowPreview()
{
	if (m_iDisplayOp != -1) {
		m_ArtCam.Close();	// Release device
	}

	// Format buffer
	if(!CreateImageBuffer())
		return;
	
	// Preview
	m_iDisplayOp = 1;
	StartPreview();
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Switch preview mode
// 
//
void CSample_MFCView::ShowStop()
{
	if(1 == m_iDisplayOp){
		if(m_bShowStop)		
			StartPreview();
		else				
			StopPreview();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Start preview
// 
//
void CSample_MFCView::StartPreview(void)
{
	m_ArtCam.Preview();
	m_ArtCam.Start();	
	GetParent()->GetMenu()->GetSubMenu(1)->ModifyMenu(
		3, MF_BYPOSITION | MF_STRING, IDM_SHOW_STOP, _T("Pause"));
	m_bShowStop = FALSE;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Stop preview
// 
//
void CSample_MFCView::StopPreview(void)
{
	m_ArtCam.Pause();
	GetParent()->GetMenu()->GetSubMenu(1)->ModifyMenu(
		3, MF_BYPOSITION | MF_STRING, IDM_SHOW_STOP, _T("Play"));
	m_bShowStop = TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Message processing during preview
//  THis is a process when you receive WM_GRAPHPAINT message
//  
LRESULT CSample_MFCView::OnGraphPaint(WPARAM wParam, LPARAM lParam)
{
	// Get data depend on a flag
	if(m_GetFlg.GetColor || m_GetFlg.ViewColor)
		m_ArtCam.GetColor(lParam, m_CDibColor.GetImage(),  m_CDibColor.GetSize());

	if(m_GetFlg.GetRaw || !m_GetFlg.ViewColor)
		m_ArtCam.GetRaw(lParam, m_CDibRaw.GetImage(), m_CDibRaw.GetSize());


	// You get temperature data by converting it in a temperature table
	//Therefore, CPU carries a load at preview mode
	if(m_GetFlg.GetTemp)
		m_ArtCam.GetTemperature	(lParam, (float*)m_CDibTemp.GetImage(),	  m_CDibTemp.GetSize() );

	// Update status bar
	CMainFrame *pFrame = (CMainFrame *)AfxGetMainWnd(); 
	CString fps;
	
	// Get FPS and indicate
	fps.Format(_T("FPS: %.1f"), m_ArtCam.GetParam(PARAM_FPS) /100.0);
	pFrame->m_wndStatusBar.SetPaneText(STATUS_FPS, fps);

	POINT point;
	GetCursorPos(&point);
	ScreenToClient(&point);
	UpdateStatusBar(point);


	// Call WM_PAINT and update a screen
	RECT rc;
	GetClientRect(&rc);
	InvalidateRect(&rc, FALSE);
	
	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Show scanned image
//  
//
void CSample_MFCView::OnPaint() 
{
	CPaintDC dc(this);

	// Do not start processing till get an image
	if(-1 == m_iDisplayOp)
		return;


	// Convert when it is monochrome mode
	if(!m_GetFlg.ViewColor)
		ConvertDataToMono();


	// Draw if an image is actual size
	if(1 == m_iStretchSize){
		if(m_GetFlg.ViewColor)
			BitBlt(dc.m_hDC, 0, 0, m_CDibColor.GetWidth(), m_CDibColor.GetHeight(), m_CDibColor.GetDC(), 0, 0, SRCCOPY);
		else
			BitBlt(dc.m_hDC, 0, 0, m_CDibMono.GetWidth(), m_CDibMono.GetHeight(), m_CDibMono.GetDC(), 0, 0, SRCCOPY);
	// Magnify and draw if magnification rate is specified
	}else{
		if(m_GetFlg.ViewColor)
			StretchBlt(dc.m_hDC, 0, 0, m_CDibColor.GetWidth() * m_iStretchSize, m_CDibColor.GetHeight() * m_iStretchSize, 
			m_CDibColor.GetDC(), 0, 0, m_CDibColor.GetWidth(), m_CDibColor.GetHeight(), SRCCOPY);
		else
			StretchBlt(dc.m_hDC, 0, 0, m_CDibMono.GetWidth() * m_iStretchSize, m_CDibMono.GetHeight() * m_iStretchSize, 
			m_CDibMono.GetDC(), 0, 0, m_CDibMono.GetWidth(), m_CDibMono.GetHeight(), SRCCOPY);
	}	

}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Process to show monochrome image
//	
//
void CSample_MFCView::ConvertDataToMono() 
{
	LONG Min   = (LONG)m_ArtCam.GetParam(PARAM_COLOR_MIN);	// Minimum value of data
	LONG Max   = (LONG)m_ArtCam.GetParam(PARAM_COLOR_MAX);	// Maximum value of data
	LONG Range = Max - Min;									// Range of data


	// Raw data(WORD)  View monochrome image(BYTE)
	LPWORD pSrcImage = (LPWORD)m_CDibRaw.GetImage();
	LPBYTE pDstImage = m_CDibMono.GetImage();

	// Convert data in a range of 256 gradations
	for(int i=0; i<m_CDibMono.GetSize(); i++){
		LONG Data = (LONG)((pSrcImage[i] - Min) * 255 / Range);
		if(Data > 255) Data = 255;
		else if(Data < 0) Data = 0;
		pDstImage[i] = (BYTE)(Data > 255 ? 255 : Data);
	}
}




//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Get a data by snapshot
// 
//
void CSample_MFCView::OnShowSnapShot()
{
	if (m_iDisplayOp != -1 || 1 == m_Param.NowRecording) {
		m_ArtCam.Close();	// Release device
	}

	CreateImageBuffer();

	DWORD FrameID = m_ArtCam.OneShot();

	if(m_GetFlg.GetColor || m_GetFlg.ViewColor)
		m_ArtCam.GetColor(FrameID, m_CDibColor.GetImage(), m_CDibColor.GetSize());

	if(m_GetFlg.GetRaw || !m_GetFlg.ViewColor)
		m_ArtCam.GetRaw(FrameID, m_CDibRaw.GetImage(), m_CDibRaw.GetSize());

	if(m_GetFlg.GetTemp)
		m_ArtCam.GetTemperature(FrameID, (PFLOAT)m_CDibTemp.GetImage(), m_CDibTemp.GetSize());

	m_iDisplayOp = 2;

	// Call WM_PAINT
	RECT rc;
	GetClientRect(&rc);
	InvalidateRect(&rc, FALSE);					

}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Get coordination of a mouse pointer on an image
//  Please refer this process when you want to get specified coordination data
//
void CSample_MFCView::UpdateStatusBar(POINT point)
{
	// Get relative coordination of a mouse pointer on an image
	// Ignore if a pointer is out of an image
	if(0 >  point.x || point.x >= m_ArtCam.Width() * m_iStretchSize ||  0 > point.y || point.y >= m_ArtCam.Height() * m_iStretchSize)
		return;
		

	// Do not start processing if data range is not formatted
	if(-1 != m_iDisplayOp){

		// Indicate coordination
		CString pos, raw, temp;
		pos.Format(_T("X: %d Y: %d"), point.x / m_iStretchSize , point.y / m_iStretchSize);

		CMainFrame *pFrame = (CMainFrame *)AfxGetMainWnd(); 
		pFrame->m_wndStatusBar.SetPaneText(STATUS_POINTER, pos);

		// Convert a position of received data on buffer to pointer's offset
		UINT DataOffset = (m_ArtCam.Height() - point.y / m_iStretchSize - 1) * m_ArtCam.Width() + point.x / m_iStretchSize;

		// Double check to avoid violation of memory access
		if(DataOffset <= (UINT)(m_CDibRaw.GetWidth() * m_CDibRaw.GetHeight()))
		{
			// Indicate RAW data
			if(m_GetFlg.GetRaw){
				LPWORD pData = (LPWORD)m_CDibRaw.GetImage();
				WORD Data = pData[DataOffset];

				raw.Format(_T("DATA: %d"), Data);
				pFrame->m_wndStatusBar.SetPaneText(STATUS_RAWDATA, raw);
			}else{
				raw.Format(_T(""));
				pFrame->m_wndStatusBar.SetPaneText(STATUS_RAWDATA, raw);
			}

			// Indicate temperature data
			if(m_GetFlg.GetTemp){
				PFLOAT pTemp = (PFLOAT)m_CDibTemp.GetImage();
				float Temp = pTemp[DataOffset];

				temp.Format(_T("Temperature: %.1f"), Temp);
				pFrame->m_wndStatusBar.SetPaneText(STATUS_TEMPDATA, temp);
			}else{
				temp.Format(_T(""));
				pFrame->m_wndStatusBar.SetPaneText(STATUS_TEMPDATA, temp);
			}
		}
	}
}


void CSample_MFCView::OnMouseMove(UINT nFlags, CPoint point)
{
	UpdateStatusBar((POINT)point);
	CView::OnMouseMove(nFlags, point);
}

void CSample_MFCView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	// 
	// 
	if(0 >  point.x || point.x >= m_ArtCam.Width() * m_iStretchSize ||  0 > point.y || point.y >= m_ArtCam.Height() * m_iStretchSize)
		return;

	// 
	if(-1 != m_iDisplayOp){

		// 
		UINT DataOffset = (m_ArtCam.Height() - point.y / m_iStretchSize - 1) * m_ArtCam.Width() + point.x / m_iStretchSize;

		// 
		if(DataOffset <= (UINT)(m_CDibRaw.GetWidth() * m_CDibRaw.GetHeight()))
		{
			// 
			if(m_GetFlg.GetRaw && m_GetFlg.GetTemp){
				LPWORD pData = (LPWORD)m_CDibRaw.GetImage();
				WORD Data = pData[DataOffset];

				PFLOAT pTemp = (PFLOAT)m_CDibTemp.GetImage();
				float Temp = pTemp[DataOffset];

				COffsetDlg	COD;
				COD.m_iData = Data;
				COD.m_fData = Temp;

				if(IDOK == COD.DoModal())
				{
					if(COD.m_iData){
						double thermo = m_ArtCam.GetThermoTable(COD.m_iData);
						double offset = COD.m_fData - thermo;

						for(int i=0; i<16384; i++)
							m_ArtCam.SetOffsetValue(i,  offset);

						m_OffsetValue.Data		 = COD.m_iData;					
					}

				}
			}
		}
	}


	CView::OnMouseMove(nFlags, point);
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Error processing
//  Process for message of WM_ERROR
//
LRESULT CSample_MFCView::OnError(WPARAM wParam, LPARAM lParam)
{
	CMainFrame *pFrame = (CMainFrame *)AfxGetMainWnd(); 
	CString err;

	// Get error and indicate on a status bar
	switch(lParam){
		case ARTCAMSDK_NOERROR:			err.Format(_T("%s"), SZERRORLIST[0]);	break;
		case ARTCAMSDK_NOT_INITIALIZE:	err.Format(_T("%s"), SZERRORLIST[1]);	break;
		case ARTCAMSDK_DISABLEDDEVICE:	err.Format(_T("%s"), SZERRORLIST[2]);	break;
		case ARTCAMSDK_CAPTURE:			err.Format(_T("%s"), SZERRORLIST[3]);	break;
		case ARTCAMSDK_CAPTURELOST:		err.Format(_T("%s"), SZERRORLIST[4]);	break;
		case ARTCAMSDK_PARAM:			err.Format(_T("%s"), SZERRORLIST[5]);	break;
		case ARTCAMSDK_OUTOFMEMORY:		err.Format(_T("%s"), SZERRORLIST[6]);	break;
		case ARTCAMSDK_CREATETHREAD:	err.Format(_T("%s"), SZERRORLIST[7]);	break;
		case ARTCAMSDK_CAMERASET:		err.Format(_T("%s"), SZERRORLIST[8]);	break;
		case ARTCAMSDK_FILENOTFOUND:	err.Format(_T("%s"), SZERRORLIST[9]);	break;
		case ARTCAMSDK_UNSUPPORTED:		err.Format(_T("%s"), SZERRORLIST[10]);	break;
		case ARTCAMSDK_UNKNOWN:
		default:						err.Format(_T("%s"), SZERRORLIST[11]);	break;
	}

	pFrame->m_wndStatusBar.SetPaneText(STATUS_ERROR, err);

	return 0L;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Call saved still image dialog
//  (SaveFileDlg.cpp)
//
void CSample_MFCView::FileSave(void)
{
	CSaveFileDlg SID;
	SID.m_pArtCam = &m_ArtCam;
	SID.m_pDoc = GetDocument();
	SID.DoModal();	
	
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Call saved slide images dialog
//  (SaveRecDlg.cpp)
//
void CSample_MFCView::RecSave(void)
{	
	CSaveRecDlg SID;
	SID.m_pArtCam = &m_ArtCam;
	SID.m_pParam  = &m_Param;
	SID.m_pDoc    = GetDocument();
	SID.DoModal();
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Call camera setting dialog
//  (ParamSetupDlg.cpp)
//
void CSample_MFCView::OpenParamDlg(void)
{
	CParamSetupDlg SID;
	SID.m_pArtCam = &m_ArtCam;
	SID.m_pParam  = &m_Param;
	SID.DoModal();
}

//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Call read saved data dialog
//  (ReplayDlg.cpp)
//
void CSample_MFCView::OnReadData()
{
	CReplayDlg SID;
	SID.m_pArtCam = &m_ArtCam;
	SID.m_pDoc    = GetDocument();
	SID.DoModal();
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Call for Address Setting dialogue
//  (IpSetDlg.cpp)
//
void CSample_MFCView::OnSetip()
{
	BYTE Add[10] = {0};
	CIpSetDlg	CIPD;
	CIPD.m_CameraType = m_CameraType;
	if(IDOK == CIPD.DoModal())
	{
		switch(m_CameraType)
		{
			case ARTCAM_CAMERATYPE_320LAN:
			case ARTCAM_CAMERATYPE_640LAN:
			case ARTCAM_CAMERATYPE_320LAN2:
			case ARTCAM_CAMERATYPE_640LAN2:
			case ARTCAM_CAMERATYPE_320LAN3:
			case ARTCAM_CAMERATYPE_640LAN3:
				m_ArtCam.Close();
				CIPD.GetAddress(Add, 4);
				if(m_ArtCam.SetIpAddress(Add[0], Add[1], Add[2], Add[3])){
					if (m_ArtCam.IsInit()) {
						m_Initialized = TRUE;
						m_ArtCam.SetParam(PARAM_NUC_AUTO, 0);
					}
				}
			break;
			case ARTCAM_CAMERATYPE_320GIGE:
			case ARTCAM_CAMERATYPE_640GIGE:
				m_ArtCam.Close();
				CIPD.GetAddress(Add, 6);
				if(m_ArtCam.ConnectToCamera(Add)){
					if (m_ArtCam.IsInit()) {
						m_ArtCam.OneShot();	
					}
				}
			break;
		}
	}
}



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Magunification rate setting
//  
//
void CSample_MFCView::OnDisplayX1(){	m_iStretchSize = 1;		InvalidateRect(FALSE);	}
void CSample_MFCView::OnDisplayX2(){	m_iStretchSize = 2;		InvalidateRect(FALSE);	}
void CSample_MFCView::OnDisplayX3(){	m_iStretchSize = 3;		InvalidateRect(FALSE);	}



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Device choosing menu
//  
//
void CSample_MFCView::OnDevice01()	{ OnDeviceChange(0); }
void CSample_MFCView::OnDevice02()	{ OnDeviceChange(1); }
void CSample_MFCView::OnDevice03()	{ OnDeviceChange(2); }
void CSample_MFCView::OnDevice04()	{ OnDeviceChange(3); }
void CSample_MFCView::OnDevice05()	{ OnDeviceChange(4); }
void CSample_MFCView::OnDevice06()	{ OnDeviceChange(5); }
void CSample_MFCView::OnDevice07()	{ OnDeviceChange(6); }
void CSample_MFCView::OnDevice08()	{ OnDeviceChange(7); }



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Category to show on a status bar
//  
//
void CSample_MFCView::OnShowRaw()
{
	m_GetFlg.GetRaw = !m_GetFlg.GetRaw;
	if(m_GetFlg.GetRaw)
		GetParent()->GetMenu()->GetSubMenu(1)->CheckMenuItem(IDM_SHOW_RAW, MF_CHECKED);
	else
		GetParent()->GetMenu()->GetSubMenu(1)->CheckMenuItem(IDM_SHOW_RAW, MF_UNCHECKED);
}

void CSample_MFCView::OnShowTemp()
{
	m_GetFlg.GetTemp = !m_GetFlg.GetTemp;
	if(m_GetFlg.GetTemp)
		GetParent()->GetMenu()->GetSubMenu(1)->CheckMenuItem(IDM_SHOW_TEMP, MF_CHECKED);
	else
		GetParent()->GetMenu()->GetSubMenu(1)->CheckMenuItem(IDM_SHOW_TEMP, MF_UNCHECKED);
}


void CSample_MFCView::OnTimer(UINT_PTR nIDEvent)
{
	if(m_Timer == nIDEvent){
		BOOL Error = FALSE;
		float IntTemp = m_ArtCam.GetInternalTemperature(&Error);
		if(Error){
			CString temp;
			temp.Format(_T("Internal Temperature: %.1f"), IntTemp);
			CMainFrame *pFrame = (CMainFrame *)AfxGetMainWnd(); 
			pFrame->m_wndStatusBar.SetPaneText(STATUS_INTTEMP, temp);
		}
	}

	CView::OnTimer(nIDEvent);
}
